@@ -3,33 +3,29 @@
 use strict;
 use warnings;
 use lib ('../lib/', 'lib/');
-use aliased 'AI::Prolog::Parser';
-use aliased 'AI::Prolog::Term';
-use aliased 'AI::Prolog::Engine';
 
-my $term   = Term->new("steals(badguy,X).");
-my $engine = Engine->new(
-    $term,
-    Parser->consult(
-        thief_prog(),
-        {},
-    )
-);
-Engine->trace(1);
-print $engine->results;
+use aliased 'AI::Prolog';
+my $logic = Prolog->new(thief_prog());
+$logic->query('steals("Bad guy", STUFF, VICTIM)');
+Prolog->trace(1);
+while (my $results = $logic->results) {
+    printf "Bad guy steals %s from %s\n",
+        $results->STUFF, $results->VICTIM;
+}
 
 sub thief_prog {
     return <<'    END_PROG';
-    steals(PERP, STUFF) :-
+    steals(PERP, STUFF, VICTIM) :-
         thief(PERP),
         valuable(STUFF),
         owns(VICTIM,STUFF),
         not(knows(PERP,VICTIM)).
-    thief(badguy).
+    thief("Bad guy").
     valuable(gold).
     valuable(rubies).
     owns(merlyn,gold).
     owns(ovid,rubies).
+    owns(kudra, gold).
     knows(badguy,merlyn).
     END_PROG
 }
